import datetime

import numpy as np
import scipy
from datasets import load_dataset
from minio import Minio
from hashlib import sha1

_TEMP_WAV_FILE = '_temp_file.wav'


def process_numpy_array_to_md5_hash(array: np.ndarray) -> str:
    return sha1(array).hexdigest()


def save_arr(data: np.ndarray):
    scipy.io.wavfile.write(_TEMP_WAV_FILE, 48000, data)


def get_minio_client() -> Minio:
    return Minio('minio-asr-benchmarks.theliver.pl', 'minio_user', 'eUxzEQbyYPdzrLxuvvethSbk18kB2s7G')


def upload_single_commonvoice(lang: str, dataset_app_name: str):
    ds = load_dataset("mozilla-foundation/common_voice_11_0", lang)
    ds = ds['test']
    minio = get_minio_client()
    index = 1
    print(ds[0])
    for it in ds:
        save_arr(it['audio']['array'])
        item_id = process_numpy_array_to_md5_hash(it['audio']['array'])
        print(datetime.datetime.now().isoformat(), f'{index} / {len(ds)}   {item_id}')
        minio.put_object('dataset-audio', f'{dataset_app_name}/{item_id}.wav', open(_TEMP_WAV_FILE, 'rb'),
                         len(open(_TEMP_WAV_FILE, 'rb').read()))
        index += 1


def upload_fleurs_dataset(hf_name: str, hf_lang: str, app_name: str):
    fleurs_asr = load_dataset(hf_name, hf_lang)
    ds = fleurs_asr['test']
    minio = get_minio_client()
    index = 1
    print(ds[0])
    for it in ds:
        item_id = it["id"]
        print(datetime.datetime.now().isoformat(), f'{index} / {len(ds)}   {item_id}')
        data = open(it['path'], 'rb').read()
        minio.put_object('dataset-audio', f'{app_name}/{it["id"]}.wav', open(it['path'], 'rb'), len(data))
        index += 1


def upload_minds_dataset(hf_lang: str, app_name: str):
    minds = load_dataset('PolyAI/minds14', hf_lang)
    ds = minds['train']
    minio = get_minio_client()
    index = 1
    print(minds)
    print(ds[0])
    for it in ds:
        data = open(it['path'], 'rb').read()
        item_id = process_numpy_array_to_md5_hash(it['audio']['array'])
        print(datetime.datetime.now().isoformat(), f'{index} / {len(ds)}   {item_id}')
        minio.put_object('dataset-audio', f'{app_name}/{item_id}.wav', open(it['path'], 'rb'), len(data))
        index += 1


# def upload_fleurs():
# upload_fleurs_dataset('google/fleurs', 'nl_nl', 'nl_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'fr_fr', 'fr_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'de_de', 'de_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'it_it', 'it_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'pl_pl', 'pl_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'es_419', 'es_google_fleurs')
# upload_fleurs_dataset('google/fleurs', 'en_us', 'en_google_fleurs')


def upload_minds():
    # upload_minds_dataset('nl-NL', 'nl_minds14')
    # upload_minds_dataset('fr-FR', 'fr_minds14')
    # upload_minds_dataset('de-DE', 'de_minds14')
    # upload_minds_dataset('it-IT', 'it_minds14')
    # upload_minds_dataset('pl-PL', 'pl_minds14')
    upload_minds_dataset('es-ES', 'es_minds14')
    upload_minds_dataset('en-US', 'en_minds14')


def upload_common_voice():
    upload_single_commonvoice('nl', 'nl_common_voice')
    upload_single_commonvoice('fr', 'fr_common_voice')
    upload_single_commonvoice('de', 'de_common_voice')
    upload_single_commonvoice('it', 'it_common_voice')
    upload_single_commonvoice('pl', 'pl_common_voice')
    upload_single_commonvoice('es', 'es_common_voice')
    upload_single_commonvoice('en', 'en_common_voice')


if __name__ == '__main__':
    # upload_fleurs()
    upload_minds()
    # upload_common_voice()
